@@ -3,13 +3,10 @@ import time
 from http.server import BaseHTTPRequestHandler
 from urllib.parse import parse_qs, unquote, urlsplit
 
-from .compat import decode_from_bytes, do_the_magic, encode_to_bytes
-from .mocket import Mocket, MocketEntry
+from httptools.parser import HttpRequestParser
 
-try:
-    from http_parser.parser import HttpParser
-except ImportError:
-    from http_parser.pyparser import HttpParser
+from .compat import ENCODING, decode_from_bytes, do_the_magic, encode_to_bytes
+from .mocket import Mocket, MocketEntry
 
 try:
     import magic
@@ -17,41 +14,69 @@ except ImportError:
     magic = None
 
 
-STATUS = dict([(k, v[0]) for k, v in BaseHTTPRequestHandler.responses.items()])
+STATUS = {k: v[0] for k, v in BaseHTTPRequestHandler.responses.items()}
 CRLF = "\r\n"
 
 
+class Protocol:
+    def __init__(self):
+        self.url = None
+        self.body = None
+        self.headers = {}
+
+    def on_header(self, name: bytes, value: bytes):
+        self.headers[name.decode("ascii")] = value.decode("ascii")
+
+    def on_body(self, body: bytes):
+        try:
+            self.body = body.decode(ENCODING)
+        except UnicodeDecodeError:
+            self.body = body
+
+    def on_url(self, url: bytes):
+        self.url = url.decode("ascii")
+
+
 class Request:
-    parser = None
-    _body = None
+    _protocol = None
+    _parser = None
 
     def __init__(self, data):
-        self.parser = HttpParser()
-        self.parser.execute(data, len(data))
-
-        self.method = self.parser.get_method()
-        self.path = self.parser.get_path()
-        self.headers = self.parser.get_headers()
-        self.querystring = parse_qs(
-            unquote(self.parser.get_query_string()), keep_blank_values=True
-        )
-        if self.querystring:
-            self.path += "?{}".format(self.parser.get_query_string())
+        self._protocol = Protocol()
+        self._parser = HttpRequestParser(self._protocol)
+        self.add_data(data)
 
     def add_data(self, data):
-        self.parser.execute(data, len(data))
+        self._parser.feed_data(data)
+
+    @property
+    def method(self):
+        return self._parser.get_method().decode("ascii")
+
+    @property
+    def path(self):
+        return self._protocol.url
+
+    @property
+    def headers(self):
+        return self._protocol.headers
+
+    @property
+    def querystring(self):
+        parts = self._protocol.url.split("?", 1)
+        if len(parts) == 2:
+            return parse_qs(unquote(parts[1]), keep_blank_values=True)
+        return {}
 
     @property
     def body(self):
-        if self._body is None:
-            self._body = decode_from_bytes(self.parser.recv_body())
-        return self._body
+        return self._protocol.body
 
     def __str__(self):
         return "{} - {} - {}".format(self.method, self.path, self.headers)
 
 
-class Response(object):
+class Response:
     headers = None
     is_file_object = False
 
@@ -149,14 +174,20 @@ class Entry(MocketEntry):
         self._match_querystring = match_querystring
 
     def collect(self, data):
+        consume_response = True
+
         decoded_data = decode_from_bytes(data)
         if not decoded_data.startswith(Entry.METHODS):
             Mocket.remove_last_request()
             self._sent_data += data
+            consume_response = False
         else:
             self._sent_data = data
+
         super(Entry, self).collect(self._sent_data)
 
+        return consume_response
+
     def can_handle(self, data):
         r"""
         >>> e = Entry('http://www.github.com/?bar=foo&foobar', Entry.GET, (Response(b'<html/>'),))
@@ -170,10 +201,8 @@ class Entry(MocketEntry):
             requestline, _ = decode_from_bytes(data).split(CRLF, 1)
             method, path, version = self._parse_requestline(requestline)
         except ValueError:
-            try:
-                return self == Mocket._last_entry
-            except AttributeError:
-                return False
+            return self is getattr(Mocket, "_last_entry", None)
+
         uri = urlsplit(path)
         can_handle = uri.path == self.path and method == self.method
         if self._match_querystring:
@@ -204,12 +233,14 @@ class Entry(MocketEntry):
         )
         if m:
             return m.group(1).upper(), m.group(2), m.group(3)
-        else:
-            raise ValueError("Not a Request-Line")
+        raise ValueError("Not a Request-Line")
 
     @classmethod
     def register(cls, method, uri, *responses, **config):
 
+        if "body" in config or "status" in config:
+            raise AttributeError("Did you mean `Entry.single_register(...)`?")
+
         default_config = dict(match_querystring=True, add_trailing_slash=True)
         default_config.update(config)
         config = default_config
@@ -240,5 +271,8 @@ class Entry(MocketEntry):
         )
 
         cls.register(
-            method, uri, response, match_querystring=match_querystring,
+            method,
+            uri,
+            response,
+            match_querystring=match_querystring,
         )
